#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "fitsio.h"

#define LINEPIX 4096
#define MAXCHAR 4096
#define MAXHDR 4096
#define HDRSIZE 100

float median(float x[], int n);
void sort(int n, float ra[]);

main(int argc, char **argv)
{
  FILE *fp;
  fitsfile *fptr1,*fptr2;
  int status, nfound, anynull;
  long fpixel, npixels;
  int i,j;
  float nullval; 
  char openfile[100],savefile[100],paramfile[100];

  int bitpix=-32;
  long naxis=2;

  long naxes[2]={LINEPIX, LINEPIX};
  static float buffer[LINEPIX],pix[LINEPIX][LINEPIX];
  long nbuffer=LINEPIX;

  char line[MAXCHAR];
 
  int nkeys;
  char *hdrptr[MAXHDR];

 
  if(argc !=3){
    fprintf(stderr,"Usage : %s (input file) (output file)\n",argv[0]);
    exit(1);
  }

  sprintf(openfile, "%s",argv[1]);
  sprintf(savefile, "%s",argv[2]);

  printf("%s \t: Read %s  ... ",argv[0],openfile);

  status=0;

  if(fits_open_file(&fptr1,openfile, READONLY, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /* Read Header Information */
  if(fits_read_keys_lng(fptr1, "NAXIS", 1, 2, naxes, &nfound, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if((naxes[0]>LINEPIX/2)||(naxes[1]>LINEPIX/2)){
    fprintf(stderr,"Image is too large\n");
    exit(0);
  }
  printf("%d x %d pixels\n",naxes[0],naxes[1]);

  if(fits_get_hdrspace(fptr1, &nkeys, NULL, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    hdrptr[i]=malloc(HDRSIZE);
    if(fits_read_record(fptr1,i+1,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  
  /* naxes[0] :  x */
  /* naxes[1] : Կ y */
  /* j :  x*/
  /* i : Կ y*/
  /* pix[y][x] */

  fpixel = 1;
  nullval = 0;
  nbuffer=naxes[0];
  for(i=0;i<naxes[1];i++){
    if (fits_read_img(fptr1, TFLOAT, fpixel, nbuffer, &nullval,
                      buffer, &anynull, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    for(j=0;j<naxes[0];j++){
      pix[i*2][j*2]=buffer[j]/4;
      pix[i*2+1][j*2]=buffer[j]/4;
      pix[i*2][j*2+1]=buffer[j]/4;
      pix[i*2+1][j*2+1]=buffer[j]/4;
    }
    fpixel+=nbuffer;
  }

  if(fits_close_file(fptr1, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("Read OK\n");

  naxes[0]*=2;
  naxes[1]*=2;
  nbuffer=naxes[0];

  printf("\t: Write %s  ... ",savefile);
  unlink(savefile);

  printf("%d x %d pixels\n",naxes[0],naxes[1]);
  status=0;

  if (fits_create_file(&fptr2, savefile, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    if(fits_write_record(fptr2,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  if(fits_update_key_lng(fptr2, "NAXIS1", naxes[0], "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }
  if(fits_update_key_lng(fptr2, "NAXIS2", naxes[1], "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_lng(fptr2, "WCSDIM", 2, "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_lng(fptr2, "LTV1", 0, "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_lng(fptr2, "LTV2", 0,  "&", &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_lng(fptr2, "LTM1_1", 2, "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_lng(fptr2, "LTM2_2", 2, "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_str(fptr2, "WAT0_001", "system=physical", "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_str(fptr2, "WAT1_001", "wtype=linear", "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if(fits_update_key_str(fptr2, "WAT2_001", "wtype=linear", "&" ,&status)){
    fits_report_error(stderr,status);
    return(status);
  }

  fpixel=1;
  for(i=0;i<naxes[1];i++){
    if (fits_write_img(fptr2, TFLOAT, fpixel, nbuffer, pix[i], &status)) 
      fits_report_error(stderr,status);
    fpixel+=nbuffer;
  }

  if (fits_close_file(fptr2, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");

}
